@@ -49,7 +49,8 @@ With this module you can manipulate files.
    owner  => "root",
    group  => "root",
    mode  => 400,
-   on_change => sub { say "File was changed."; };
+   on_change => sub { say shift, " was changed."; },
+   on_no_change => sub { say shift, " wasn't changed."; };
 
 
 =head1 EXPORTED FUNCTIONS
@@ -62,7 +63,7 @@ use strict;
 use warnings;
 use Fcntl;
 
-our $VERSION = '1.12.1.3'; # TRIAL VERSION
+our $VERSION = '1.12.2.1'; # TRIAL VERSION
 
 require Rex::Exporter;
 use Data::Dumper;
@@ -297,7 +298,8 @@ This function is the successor of I<install file>. Please use this function to u
      owner  => "user",
      group  => "group",
      mode   => 700,
-     on_change => sub { say "Something was changed." };
+     on_change => sub { say "Something was changed." },
+     on_no_change => sub { say "Nothing has changed." };
  
    file "/etc/motd",
      content => `fortune`;
@@ -451,8 +453,9 @@ sub file {
   Rex::get_current_connection()->{reporter}
     ->report_resource_start( type => "file", name => $file );
 
-  my $need_md5  = ( $option->{"on_change"} && !$is_directory ? 1 : 0 );
-  my $on_change = $option->{"on_change"} || sub { };
+  my $need_md5     = ( $option->{"on_change"} && !$is_directory ? 1 : 0 );
+  my $on_change    = $option->{"on_change"}    || sub { };
+  my $on_no_change = $option->{"on_no_change"} || sub { };
 
   my $__ret = { changed => 0 };
 
@@ -740,6 +743,11 @@ sub file {
   if ( $__ret->{changed} == 1 && $on_change_done == 0 ) {
     &$on_change($file);
   }
+  elsif ( $__ret->{changed} == 0 ) {
+    Rex::Logger::debug(
+      "File $file has not been changed... Running on_no_change");
+    &$on_no_change($file);
+  }
 
   #### check and run after hook
   Rex::Hook::run_hook( file => "after", $file, %{$option}, $__ret );
@@ -964,7 +972,7 @@ OUT:
 
 =head2 delete_lines_according_to($search, $file, @options)
 
-This is the successor of the delete_lines_matching() function. This function also allows the usage of an on_change hook.
+This is the successor of the delete_lines_matching() function. This function also allows the usage of on_change and on_no_change hooks.
 
 It will search for $search in $file and remove the found lines. If on_change hook is present it will execute this if the file was changed.
 
@@ -981,8 +989,9 @@ sub delete_lines_according_to {
   my ( $search, $file, @options ) = @_;
   $file = resolv_path($file);
 
-  my $option    = {@options};
-  my $on_change = $option->{on_change} || undef;
+  my $option       = {@options};
+  my $on_change    = $option->{on_change}    || undef;
+  my $on_no_change = $option->{on_no_change} || undef;
 
   my ( $old_md5, $new_md5 );
 
@@ -992,11 +1001,14 @@ sub delete_lines_according_to {
 
   delete_lines_matching( $file, $search );
 
-  if ($on_change) {
+  if ( $on_change || $on_no_change ) {
     $new_md5 = md5($file);
 
     if ( $old_md5 ne $new_md5 ) {
-      &$on_change($file);
+      &$on_change($file) if $on_change;
+    }
+    else {
+      &$on_no_change($file) if $on_no_change;
     }
   }
 
@@ -1043,8 +1055,11 @@ found, it will be updated. Otherwise, it will be appended.
      line  => "mygroup:*:100:myuser3,myuser4",
      regexp => qr{^mygroup},
      on_change => sub {
-                say "file was changed, do something.";
-              };
+       say "file was changed, do something.";
+     },
+     on_no_change => sub {
+       say "file was not changed, do something.";
+     };
  };
 
 =cut
@@ -1061,7 +1076,7 @@ sub _append_or_update {
   my ( $new_line, @m );
 
   # check if parameters are in key => value format
-  my ( $option, $on_change );
+  my ( $option, $on_change, $on_no_change );
 
   Rex::get_current_connection()->{reporter}
     ->report_resource_start( type => $action, name => $file );
@@ -1082,17 +1097,22 @@ sub _append_or_update {
     elsif ( ref $option->{regexp} eq "ARRAY" ) {
       @m = @{ $option->{regexp} };
     }
-    $on_change = $option->{on_change} || undef;
+    $on_change    = $option->{on_change}    || undef;
+    $on_no_change = $option->{on_no_change} || undef;
     1;
   } or do {
     ( $new_line, @m ) = @_;
 
-    # check if something in @m (the regexpes) is named on_change
-    for ( my $i = 0 ; $i < $#m ; $i++ ) {
-      if ( $m[$i] eq "on_change" && ref( $m[ $i + 1 ] ) eq "CODE" ) {
-        $on_change = $m[ $i + 1 ];
-        splice( @m, $i, 2 );
-        last;
+    # check if something in @m (the regexpes) is named on_change or on_no_change
+    for my $option ( [ on_change => \$on_change ],
+      [ on_no_change => \$on_no_change ] )
+    {
+      for ( my $i = 0 ; $i < $#m ; $i++ ) {
+        if ( $m[$i] eq $option->[0] && ref( $m[ $i + 1 ] ) eq "CODE" ) {
+          ${ $option->[1] } = $m[ $i + 1 ];
+          splice( @m, $i, 2 );
+          last;
+        }
       }
     }
   };
@@ -1130,32 +1150,47 @@ sub _append_or_update {
         $match = qr{$match};
       }
       if ( $content->[$line] =~ $match ) {
-        return 0 if $action eq 'append_if_no_such_line';
-        $content->[$line] = "$new_line";
         $found = 1;
+        last if $action eq 'append_if_no_such_line';
+        $content->[$line] = "$new_line";
       }
     }
   }
 
-  push @$content, "$new_line" unless $found;
+  my $new_md5;
+  if ( $action eq 'append_if_no_such_line' && $found ) {
+    $new_md5 = $old_md5;
+  }
+  else {
+    push @$content, "$new_line" unless $found;
 
-  file $file,
-    content => join( "\n", @$content ),
-    owner   => $stat{uid},
-    group   => $stat{gid},
-    mode    => $stat{mode};
+    file $file,
+      content => join( "\n", @$content ),
+      owner   => $stat{uid},
+      group   => $stat{gid},
+      mode    => $stat{mode};
+    $new_md5 = md5($file);
+  }
 
-  my $new_md5 = md5($file);
-
-  if ($on_change) {
+  if ( $on_change || $on_no_change ) {
     if ( $old_md5 && $new_md5 && $old_md5 ne $new_md5 ) {
-      $old_md5 ||= "";
+      if ($on_change) {
+        $old_md5 ||= "";
+        $new_md5 ||= "";
+
+        Rex::Logger::debug("File $file has been changed... Running on_change");
+        Rex::Logger::debug("old: $old_md5");
+        Rex::Logger::debug("new: $new_md5");
+        &$on_change($file);
+      }
+    }
+    elsif ($on_no_change) {
       $new_md5 ||= "";
 
-      Rex::Logger::debug("File $file has been changed... Running on_change");
-      Rex::Logger::debug("old: $old_md5");
-      Rex::Logger::debug("new: $new_md5");
-      &$on_change($file);
+      Rex::Logger::debug(
+        "File $file has not been changed (md5 $new_md5)... Running on_no_change"
+      );
+      &$on_no_change($file);
     }
   }
 
@@ -1278,6 +1313,8 @@ Search some string in a file and replace it.
     multiline => TRUE;
  };
 
+Like similar file management commands, it also supports C<on_change> and C<on_no_change> hooks.
+
 =cut
 
 sub sed {
@@ -1295,7 +1332,8 @@ sub sed {
     $options = {@option};
   }
 
-  my $on_change = $options->{"on_change"} || undef;
+  my $on_change    = $options->{"on_change"}    || undef;
+  my $on_no_change = $options->{"on_no_change"} || undef;
 
   my @content;
 
@@ -1315,11 +1353,12 @@ sub sed {
 
   my $ret = file(
     $file,
-    content   => join( "\n", @content ),
-    on_change => $on_change,
-    owner     => $stat{uid},
-    group     => $stat{gid},
-    mode      => $stat{mode}
+    content      => join( "\n", @content ),
+    on_change    => $on_change,
+    on_no_change => $on_no_change,
+    owner        => $stat{uid},
+    group        => $stat{gid},
+    mode         => $stat{mode}
   );
 
   Rex::get_current_connection()->{reporter}
