@@ -28,7 +28,7 @@ package Rex::Commands::MD5;
 use strict;
 use warnings;
 
-our $VERSION = '1.12.1.3'; # TRIAL VERSION
+our $VERSION = '1.12.2.1'; # TRIAL VERSION
 
 use Rex::Logger;
 require Rex::Commands;
@@ -107,7 +107,7 @@ sub _binary_md5 {
   my $exec = Rex::Interface::Exec->create;
 
   if ( Rex::Commands::Run::can_run('md5') ) {
-    ($md5) = $exec->exec("md5 '$file'") =~ qr{\s+=\s+([a-f0-9]{32})};
+    ($md5) = $exec->exec("md5 '$file'") =~ qr{\s+=\s+([a-f0-9]{32})\s*$};
   }
   elsif ( Rex::Commands::Run::can_run('md5sum') ) {
     ($md5) = $exec->exec("md5sum '$file'") =~ qr{^\\?([a-f0-9]{32})\s+};
